/* **********************************************************
 * Copyright 2006 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

/*
 * stubs.c
 *
 * Contains stubs and helper functions.
 */

/* Must come before any kernel header file. */
#include "driver-config.h"

#include "kernelStubs.h"
#include "module.h"
#include "vm_assert.h"

/*
 *----------------------------------------------------------------------
 *
 * Debug --
 *
 *    If debugging is enabled, output debug information.
 *
 * Result
 *    None
 *
 * Side-effects
 *    None
 *
 *----------------------------------------------------------------------
 */

void
Debug(char const *fmt, // IN: Format string
      ...)             // IN: Arguments
{
   va_list args;
   int numBytes;
   static char out[128];

   va_start(args, fmt);
   numBytes = Str_Vsnprintf(out, sizeof out, fmt, args);
   va_end(args);

   if (numBytes > 0) {
      LOG(6, (KERN_DEBUG "VMware hgfs: %s", out));
   }
}


/*
 *----------------------------------------------------------------------
 *
 * Log --
 *
 *    Needs to be defined.
 *
 * Result
 *    None
 *
 * Side-effects
 *    None
 *
 *----------------------------------------------------------------------
 */
void
Log(const char *string, ...)
{
   // do nothing.
}
